# encoding: utf8


import datetime
import os.path
import platform
import signal
import subprocess
import sys
import threading
import time


started_processes = []
finished_processes = []
acquired_license_count = 0
ctrl_c_detected = False

squish_dir = os.path.dirname(os.path.dirname(sys.executable))
if platform.system() != "Windows":
    squish_dir = os.path.dirname(squish_dir)


def main():
    # Checked for in output:
    print("license acquired")
    # Required:
    sys.stdout.flush()
    snooze(45)


def main_python():
    if len(sys.argv) != 2:
        print_error_usage_and_exit("Exactly 1 parameter must be specified", 1)


    if hasattr(signal, "SIGBREAK"):
        signal.signal(signal.SIGBREAK, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)


    script_dir = os.path.dirname(sys.argv[0])
    if len(script_dir) == 0:
        script_dir = "."


    competing_test_file = os.path.join(script_dir, "test.js")
    if os.path.exists(competing_test_file):
        print_error_usage_and_exit(f"File \"{competing_test_file}\" exists and must be removed. Or move test.py into a different folder.", 2)


    squishrunner_cmd = squishrunner_command(script_dir)


    kill_all_squishrunners()


    max_license_check_count = int(sys.argv[1])
    start_squishrunners(max_license_check_count, script_dir, squishrunner_cmd)


    print()
    print(f"NOTE: Executing squishrunners with command: {' '.join(squishrunner_cmd)}")
    print()


    print(f"WARNING: If the Squish IDE is running on this computer, only 1 (execution) license can be acquired on this computer.")
    print()


    seconds_to_check = 40
    for seconds in range(seconds_to_check):
        if ctrl_c_detected:
            break

        time.sleep(1)

        if ctrl_c_detected:
            break

        printts(f"Acquired licenses after {seconds+1} second(s): {acquired_license_count}")
        if acquired_license_count == max_license_check_count:
            break

        running_processes = []
        for p in started_processes:
            if p.poll() is None:
                running_processes.append(p)
        if len(running_processes) == 0 and not ctrl_c_detected:
            printts(f"ERROR: All squishrunners exited unexpectedly. Aborting.")
            break


    for p in started_processes:
        p.terminate()
    kill_all_squishrunners()


def squishrunner_command(script_dir):
    squishrunner = f"{squish_dir}/bin/squishrunner.exe"
    if platform.system() != "Windows":
        squishrunner = f"{squish_dir}/bin/squishrunner"
    return [squishrunner, "--testcase", script_dir]


def start_squishrunners(max_license_check_count, script_dir, squishrunner_cmd):
    cmd = squishrunner_command(script_dir)
    for i in range(max_license_check_count):
        t = threading.Thread(target=read_command_output, args=(squishrunner_cmd,))
        t.daemon = True
        t.start()

    # Wait for start of all squishrunner processes:
    while len(started_processes) < max_license_check_count:
        time.sleep(0.1)


def printts(s):
    print(f"{datetime.datetime.strftime(datetime.datetime.now(), '%Y-%m-%d-%H:%M:%S')} {s}")


def read_command_output(squishrunner_cmd):
    global acquired_license_count

    p = subprocess.Popen(squishrunner_cmd, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
    started_processes.append(p)

    try:
        while True:
            s = p.stdout.readline().decode()
            if "license acquired" in s:
                acquired_license_count += 1
                finished_processes.append(p)
    except:
        finished_processes.append(p)


def print_error_usage_and_exit(error_message, exit_code):
    print(f"ERROR: {error_message}")
    print()
    print(f"USAGE:")
    print(f" Unix:")
    print(f"  \"SQUISH_DIR/python3/bin/python\" test.py upper_license_check_limit")
    print()
    print(f" Windows:")
    print(f"  \"SQUISH_DIR\\python3\\python\" test.py upper_license_check_limit")
    print()
    print(f" (Replace \"SQUISH_DIR\" and \"path\\to\\this\" as needed.)")
    print()
    print(f"EXAMPLE:")
    print(f" \"C:\\Users\\myuser\\squish-7.1.1-qt65x-msvc142-x64\\python3\\python\" \"C:\\Users\\myuser\\test.py\" 10")
    print()
    print(f"WARNING:")
    print(f" If the Squish IDE is running on this computer, only 1 (execution) license can be acquired on this computer.")
    print()
    sys.exit(exit_code)


def kill_all_squishrunners():
    try:
        subprocess.run(
            ["taskkill", "/f", "/t", "/im", "_squishrunner.exe"],
            stdout = subprocess.DEVNULL,
            stderr = subprocess.DEVNULL
        )
    except:
        pass
    try:
        subprocess.run(
            ["killall", "-9", "_squishrunner"],
            stdout = subprocess.DEVNULL,
            stderr = subprocess.DEVNULL
        )
    except:
        pass


def signal_handler(sig, frame):
    global ctrl_c_detected
    ctrl_c_detected = True
    printts("Ctrl+c detected, exiting.")
    kill_all_squishrunners()


if __name__ == "__main__":
    main_python()
